#!/usr/bin/python
# -*- coding: UTF-8 -*-
from six import iteritems

from dli.column import Column

from dli.exception import DliException
from dli.rest import ExportCommonInfo, SessionInfo
from dli.rest.rest import ApiException
from dli.result_set import ResultSet
from dli.row import Row


class SqlJob(object):
    def __init__(self, job_id, job_type, client, schema=None, rows=None, status=None):
        """

        :param str job_id:
        :param str job_type:
        :param DliClient client:
        :param list schema:
        :param list rows:
        """

        self._job_id = job_id
        self._job_type = job_type
        self._schema = []
        self._rows = []
        self._client = client   # type: DliClient
        self._status = status

        if schema is not None:
            for col in schema:
                for col_name, col_type in col.items():
                    self._schema.append(Column(col_name, col_type))

        if rows is not None and schema is not None:
            for row in rows:
                self._rows.append(Row(self._schema, row))

    @property
    def job_id(self):
        return self._job_id

    @property
    def job_type(self):
        return self._job_type

    @property
    def job_status(self):
        return self._status

    def get_status(self):
        """
        This method is used to get the job execution status from server.

        :return: str: job status
        """
        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'x_auth_token': self._client.token,
            'job_id': self._job_id
        }

        try:
            resp = self._client.rest_api.get_job_status(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        self._status = resp.status
        return resp.status

    def get_result_page(self, current_page, page_size=1000, queue_name=None):
        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'x_auth_token': self._client.token,
            'job_id': self.job_id,
            'page_size': page_size,
            'current_page': current_page
        }
        if queue_name is not None:
            kwargs.update({'queue_name': queue_name})

        try:
            resp = self._client.rest_api.get_job_result(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return resp

    def preview_result(self):
        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'x_auth_token': self._client.token,
            'job_id': self.job_id
        }

        try:
            resp = self._client.rest_api.preview_job_result(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return resp

    def get_schema(self, queue_name=None):
        """
        This method is used to view the job execution result after a job
        is executed using SQL query statements. Currently, you can only
        query execution results of jobs of the QUERY type.
        You can view the first 1000 result records only. To view all result
        records, use export_result method.

        :return: list: The list of Columns, have the name and type of the job result column.
        """

        # Sql is DCL, DDL or Query in sync mode, the result rows have got
        # from submit job API call
        if len(self._schema) == 0 and self.job_type == 'QUERY':
            resp = self.get_result_page(1, queue_name=queue_name)
            for col in resp.schema:
                for col_name, col_type in col.items():
                    self._schema.append(Column(col_name, col_type))

        return self._schema

    def get_result(self, page_size=1000, queue_name=None):
        """
        This method is used to view the job execution result after a job
        is executed using SQL query statements. Currently, you can only
        query execution results of jobs of the QUERY type.
        When the query returns more records, it is recommended to use export_result method.

        :return: ResultSet.
        """

        # Sql is DCL, DDL the result rows have got
        # from submit job API call
        if self.job_type == 'QUERY':
            return ResultSet(self, page_size=page_size, queue_name=queue_name)
        else:
            return ResultSet(self, self._rows, page_size=page_size, queue_name=queue_name)

    def export_result(self, data_path, queue_name='default', compress='none',
                      export_mode='ErrorIfExists'):
        """
        This method is used to export results returned from the query
        using SQL statements to OBS.

        :param str data_path: Path for storing the exported result (currently,
                        data can be stored on an OBS path starting with s3a only). The
                        OBS path cannot contain folders, for example, the path folder
                        in the sample request.
        :param str queue_name: Name of the queue that is specified to execute a task.
                        If no queue is specified, the default queue is used.
        :param str compress: Compression format of exported data. Currently, gzip,
                        bzip2, and deflate are supported. The default value is
                        none, indicating that data is not compressed.
        :param str export_mode: Export mode. The parameter value can be
                        ErrorIfExists or Overwrite. If export_mode is not
                        specified, this parameter is set to ErrorIfExists by default.
        :return: str Status of a job, including LAUNCHING, RUNNING,
                        FINISHED, FAILED, and CANCELLED.
        """

        exp_info = ExportCommonInfo(data_path=data_path, data_type='csv',
                                    compress=compress, queue_name=queue_name,
                                    export_mode=export_mode)
        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'x_auth_token': self._client.token,
            'job_id': self.job_id,
            'export_common_info': exp_info
        }

        try:
            resp = self._client.rest_api.store_job_result(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self._client.wait_for_finish(resp.job_id)

    def cancel_job(self):
        """
        This method is used to cancel the job on server.

        :return: bool: is success
        """
        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'x_auth_token': self._client.token,
            'job_id': self._job_id
        }
        try:
            resp = self._client.rest_api.cancel_job(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return resp


class SparkJobInfo(object):
    def __init__(self, app_file, **kwargs):
        """
        Init the SparkJobInfo instance

        :param app_file:
        :param kwargs:
        """

        all_params = ['app_file', 'app_name', 'class_name', 'jars', 'py_files', 'files',
                      'confs', 'resources', 'groups', 'modules', 'sc_type', 'args', 'catalog_name',
                      'driver_memory', 'driver_cores', 'executor_memory', 'executor_cores', 'num_executors',
                      'obs_bucket', 'auto_recovery', 'max_retry_times', 'feature', 'image']

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'" % key)
            params[key] = val

        del params['kwargs']

        self._name = params.get('app_name')
        self._file = params.get('app_file')
        self._class = params.get('class_name')
        self._jars = params.get('jars')
        self._py_files = params.get('py_files')
        self._files = params.get('files')
        self._confs = params.get('confs')
        self._resources = params.get('resources')
        self._groups = params.get('groups')
        self._sc_type = params.get('sc_types')
        self._args = params.get('args')
        self._modules = params.get('modules')
        self._catalogName = params.get('catalog_name')
        self._driverMemory = params.get('driver_memory')
        self._driverCores = params.get('driver_cores')
        self._executorMemory = params.get('executor_memory')
        self._executorCores = params.get('executor_cores')
        self._numExecutors = params.get('num_executors')
        self._obsBucket = params.get('obs_bucket')
        self._autoRecovery = params.get('auto_recovery')
        self._maxRetryTimes = params.get('max_retry_times')
        self._feature = params.get('feature')
        self._image = params.get('image')

    def get_request_body(self, queue_name):

        # build a json object of the api request body for submit spark job
        rest_request_body = {
            'file': self._file,
            'className': self._class,
            'queue': queue_name,
            'args': self._args,
            'sc_type': self._sc_type,
            'jars': self._jars,
            'pyFiles': self._py_files,
            'files': self._files,
            'conf': self._confs,
            'resources': self._resources,
            'groups': self._groups,
            'name': self._name,
            'modules': self._modules,
            'catalog_name': self._catalogName,
            'driver_memory': self._driverMemory,
            'driver_cores': self._driverCores,
            'executor_memory': self._executorMemory,
            'executor_cores': self._executorCores,
            'num_executors': self._numExecutors,
            'obs_bucket': self._obsBucket,
            'auto_recovery': self._autoRecovery,
            'max_retry_times': self._maxRetryTimes,
            'feature': self._feature,
            'image': self._image
        }

        return rest_request_body


class SparkBatchJob(object):
    def __init__(self, client, job_info):
        """
        Init the SparkBatchJob instance

        :param SessionInfo job_info: The job_info return form the server
        :param DliClient client: The object of the DLIClient
        """

        self._job_info = job_info   # type: SessionInfo
        self._client = client   # type: DliClient
        self._status = None

    @property
    def job_id(self):
        return self._job_info.id

    @property
    def status(self):
        return self._status

    def get_job_status(self):
        """
        Get Spark batch job status from dli server

        :return: str Status include 'starting','running','dead','success','recovering'

        """

        try:
            job_status = self._client.rest_api.get_batch_state(project_id=self._client.dli_info.project_id,
                                                               batch_id=self._job_info.id,
                                                               x_auth_token=self._client.token)
        except ApiException as e:
            raise DliException(api_exception=e)

        self._status = job_status.state

        return job_status.state

    def get_driver_log(self, size):
        """
        Get Spark batch job driver log from dli server

        :param size: log lenth size
        :return: list[str]

        """

        kwargs = {
            'project_id': self._client.dli_info.project_id,
            'batch_id':  self._job_info.id,
            'x_auth_token': self._client.token,
            'size': size,
            'type': "driver",
            'index': 0
        }

        try:
            session_log = self._client.rest_api.get_batch_log(**kwargs)
        except ApiException as e:
            raise DliException(api_exception=e)

        return session_log.log
